@@ -3076,20 +3076,8 @@ void editline_object_init(sObject* self)
     uobject_put(editline, "keybind", nfun);
 }
 
-void xyzsh_editline_init()
+void xyzsh_editline_history_init()
 {
-    /// editline init ///
-    gEditLine = el_init("xyzsh", stdin, stdout, stderr);
-
-    el_wset(gEditLine, EL_EDITOR, L"emacs");
-    el_wset(gEditLine, EL_SIGNAL, 1);
-    el_wset(gEditLine, EL_PROMPT_ESC, editline_prompt, '\1');
-
-    el_source(gEditLine, NULL);
-
-    editline_additional_keybind_emacs();
-    gCmdLineStack = VECTOR_NEW_MALLOC(10);
-
     /// set history size ///
     int history_size;
     char* histsize_env = getenv("XYZSH_HISTSIZE");
@@ -3126,12 +3114,29 @@ void xyzsh_editline_init()
         xstrncpy(histfile, histfile_env, PATH_MAX);
     }
 
-    gHistory = history_winit();
     HistEventW ev;
     history_w(gHistory, &ev, H_SETSIZE, history_size);
     history_w(gHistory, &ev, H_LOAD, histfile);
+}
 
+void xyzsh_editline_init()
+{
+    /// editline init ///
+    gEditLine = el_init("xyzsh", stdin, stdout, stderr);
+
+    el_wset(gEditLine, EL_EDITOR, L"emacs");
+    el_wset(gEditLine, EL_SIGNAL, 1);
+    el_wset(gEditLine, EL_PROMPT_ESC, editline_prompt, '\1');
+
+    el_source(gEditLine, NULL);
+
+    editline_additional_keybind_emacs();
+    gCmdLineStack = VECTOR_NEW_MALLOC(10);
+
+    gHistory = history_winit();
     el_wset(gEditLine, EL_HIST, history_w, gHistory);
+
+    xyzsh_editline_history_init();
 }
 
 void xyzsh_editline_final()
