@@ -8,6 +8,7 @@
 #elif defined(__LINUX__)
 #define _XOPEN_SOURCE
 #define __USE_XOPEN
+#define __USE_UNIX98
 #include <wchar.h>
 #include <pty.h>
 #include <ctype.h>
@@ -1408,3 +1409,134 @@ BOOL temulator_draw_on_curses(sTEmulator* self, WINDOW* win, int win_pos_y, int
     return TRUE;
 }
 
+static BOOL gSigChld = FALSE;
+static BOOL gSigWinch = FALSE;
+
+static void handler(int signo)
+{
+    switch (signo) {
+      case SIGCHLD:
+        gSigChld = TRUE;
+        break;
+      case SIGWINCH:
+        gSigWinch = TRUE;
+        break;
+    }
+}
+
+static int is_expired(struct timeval now, struct timeval expiry)
+{
+    return now.tv_sec > expiry.tv_sec
+        || (now.tv_sec == expiry.tv_sec && now.tv_usec > expiry.tv_usec);
+}
+
+static struct timeval const slice = { 0, 1000 * 1000 / 100 };
+
+static struct timeval timeval_add(struct timeval a, struct timeval b)
+{
+    int usec = a.tv_usec + b.tv_usec;
+    a.tv_sec += b.tv_sec;
+    while (usec > 1000 * 1000) {
+        a.tv_sec += 1;
+        usec -= 1000 * 1000;
+    }
+    a.tv_usec = usec;
+    return a;
+}
+
+
+void run_on_temulator(fTEmulator fun, void* arg)
+{
+    gSigChld = FALSE;
+    gSigWinch = FALSE;
+
+    signal(SIGCHLD, handler);
+    signal(SIGWINCH, handler);
+
+    const int maxx = mgetmaxx();
+    const int maxy = mgetmaxy();
+
+    int temulator_y = 0;
+    int temulator_x = 0;
+    int temulator_height = maxy;
+    int temulator_width = maxx;
+
+    sTEmulator* temulator = temulator_init(temulator_height, temulator_width);
+
+    temulator_open(temulator, fun, arg);
+
+    initscr();
+    start_color();
+    noecho();
+    raw();
+    nodelay(stdscr, TRUE);
+    keypad(stdscr, TRUE);
+    curs_set(0);
+    ESCDELAY=50;
+
+    temulator_init_colors();
+
+    WINDOW* term_win = newwin(temulator_height, temulator_width, temulator_y, temulator_x);
+
+    int pty = temulator->mFD;
+
+    fd_set mask, read_ok;
+    FD_ZERO(&mask);
+    FD_SET(0, &mask);
+    FD_SET(pty, &mask);
+
+    int dirty = 0;
+    struct timeval next;
+
+    gettimeofday(&next, NULL);
+    while(1) {
+        struct timeval tv = { 0, 1000 * 1000 / 100 };
+        read_ok = mask;
+
+        if(select(pty+1, &read_ok, NULL, NULL, &tv) > 0) {
+            if(FD_ISSET(pty, &read_ok)) {
+                temulator_read(temulator);
+                dirty = 1;
+            }
+        }
+
+        int key;
+        while((key = getch()) != ERR) {
+            temulator_write(temulator, key);
+            dirty = 1;
+        }
+
+        gettimeofday(&tv, NULL);
+        if(dirty && is_expired(tv, next)) {
+            temulator_draw_on_curses(temulator, term_win, temulator_y, temulator_x);
+            wrefresh(term_win);
+            dirty = 0;
+            next = timeval_add(tv, slice);
+        }
+
+        if(gSigChld) {
+            gSigChld = FALSE;
+            break;
+        }
+
+        if(gSigWinch) {
+            gSigWinch = 0;
+
+            temulator_height = mgetmaxy();
+            temulator_width = mgetmaxx();
+
+            if(temulator_width >= 10 && temulator_height >= 10) {
+                resizeterm(temulator_height, temulator_width);
+
+                wresize(term_win, temulator_height, temulator_width);
+                temulator_resize(temulator, temulator_height, temulator_width);
+
+                dirty = 1;
+            }
+        }
+    }
+
+    endwin();
+
+    temulator_final(temulator);
+}
